/* volumeBase.h

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#ifndef VOLUMEBASE_H
#define VOLUMEBASE_H

// This is the abstract base class for a number of derived Volume
// types.  A Volume is the root of a directory hierarchy, somewhere in
// the world; possibly on the local machine, and possibly elsewhere.
// The Volume knows how to retrieve files from it and, if possible,
// store them in it.

#include "referenceCount.h"
#include "dataset.h"

#include <string>

class VolumeBase : public ReferenceCount {
public:
  VolumeBase(const Dataset &data);
  virtual ~VolumeBase();

  void setup();
  static VolumeBase *make_volume(const Dataset &data);

  string get_description() const;
  string get_definition() const;
  int get_volume_id() const;

  virtual bool exists() const=0;
  virtual bool can_read_files() const=0;
  virtual bool can_store_files() const=0;

  // get_filename() returns the full pathname in the Unix directory of
  // the indicated file within this volume, iff the volume is mounted
  // and readable through normal Unix system calls.  Otherwise, it
  // returns the empty string.
  virtual string get_filename(const string &rel_filename)=0;

  // prepare_write_file() returns the full pathname in the Unix
  // directory of the indicated file-to-be within this volume, in the
  // bucket it probably will be stored in.  However, this might change
  // if we later discover we have overfilled the bucket.  Once the
  // file is written and closed, call close_write_file().
  virtual string prepare_write_file(const string &basename)=0;

  // close_write_file() is called after writing a file into the
  // volume.  It checks the bucket size, and may move the file if the
  // bucket is filled.  It returns the relative pathname of the file
  // within the volume.
  virtual string close_write_file(const string &full_pathname,
				  const string &basename)=0;

  // store_file() takes a file which already exists in the Unix
  // filesystem and moves it into the volume, wherever that may be.
  // It returns the new filename on success, empty string on failure.
  virtual string store_file(const string &full_pathname, 
			    const string &basename,
			    bool leave_orig)=0;

  // Re-fetch the parameters from the database to make sure we're
  // totally in sync with all other processes.
  virtual void read_db();

protected:
  virtual void expand_bname();

  Dataset _data;
  int _volume_id;
  string _definition;
  int _bucket;
  int _bucket_size;
  string _bname_unexpanded;
  string _bname;
  
  // This is _definition + "/" + _bname.  It might make sense for some
  // volumes, but not necessarily all.
  string _dirname;
};

#endif
