/* queryResult.h

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#ifndef QUERYRESULT_H
#define QUERYRESULT_H

#include "dataCell.h"
#include "referenceCount.h"

class Database;

// A QueryResultBase is some abstract data type returned by a
// particular database type in response to a query.  While
// QueryResultBase is itself abstract and can't be created directly,
// there exists a concrete version for defined for each Database type
// (For instance, there's a PostgresQueryResult).

// Since QueryResultBase is polymorphic, code that uses it must keep a
// pointer to an object allocated off the free store, instead of a
// local copy allocated off the stack.  But it's cumbersome to manage
// the allocation/deallocation of these things.  Thus, the class
// QueryResult is used as a proxy to a QueryResultBase pointer.

// QueryResult allocates a new QueryResultBase pointer of the
// appropriate type on construction, and deallocates it on
// destruction.  All methods are transparently passed through to the
// underlying QueryResultBase, whichever type it is.

class QueryResultBase : public ReferenceCount {
public:
  QueryResultBase(Database &db);
  virtual ~QueryResultBase();

  virtual bool success() const=0;
  virtual string get_error_message() const=0;

  virtual int get_num_columns() const=0;
  virtual string get_column_name(int c) const=0;
  virtual DataCell::Type get_column_type(int c) const=0;
  
  virtual int get_num_rows() const=0;
  virtual DataCell get_cell(int r, int c) const=0;
  
protected:
  Database &_db;
};

class QueryResult : public ReferenceCountProxy {
public:
  QueryResult(QueryResultBase *ptr = (QueryResultBase *)NULL);
  QueryResult(const QueryResult &copy);
  QueryResult &operator = (const QueryResult &copy);

  bool success() const;
  string get_error_message() const;

  int get_num_columns() const;
  string get_column_name(int c) const;
  DataCell::Type get_column_type(int c) const;
  
  int get_num_rows() const;
  DataCell get_cell(int r, int c) const;

  QueryResultBase *get_ptr() const;
};

#endif
